# -*- coding: utf-8 -*-
import unittest
from cStringIO import StringIO

from schemaconfig.django import (update_settings, get_django_settings, schemas,
    Django102Schema, SETTINGS_ENCODING)
from schemaconfig.options import (DictConfigOption, IntConfigOption,
    StringConfigOption)
from schemaconfig.parser import SchemaConfigParser, CONFIG_FILE_ENCODING
from schemaconfig.schema import Schema


class DjangoSupportTestCase(unittest.TestCase):
    def test_get_django_settings(self):
        class MySchema(Schema):
            foo = IntConfigOption()
            bar = DictConfigOption({'baz': IntConfigOption(),
                                    'BAZ': IntConfigOption()})

        expected = {'FOO': 0, 'BAR': {'baz': 0, 'BAZ': 0}}

        parser = SchemaConfigParser(MySchema())
        result = get_django_settings(parser)
        self.assertEqual(result, expected)

    def test_get_django_settings_encoding(self):
        class MySchema(Schema):
            foo = StringConfigOption()

        expected = {'FOO': u'€'.encode(SETTINGS_ENCODING)}

        config = StringIO(u'[default]\nfoo=€'.encode(CONFIG_FILE_ENCODING))
        parser = SchemaConfigParser(MySchema())
        parser.readfp(config)
        self.assertEqual(parser.values(), {'foo': u'€'})
        result = get_django_settings(parser)
        self.assertEqual(result, expected)

    def test_update_settings(self):
        class MySchema(Schema):
            foo = IntConfigOption()

        expected_env = {'FOO': 0}

        env = {}
        parser = SchemaConfigParser(MySchema())
        update_settings(parser, env)
        self.assertEqual(env, expected_env)

    def test_schemafactory_get(self):
        expected = Django102Schema
        default = schemas.get(schemas._default)

        # test get valid version
        self.assertEqual(schemas.get('1.0.2'), expected)

        # test get invalid version
        self.assertEqual(schemas.get('1.1'), default)


